@@ -3,7 +3,7 @@ package Language::Lisp::ECLs;
 use 5.008;
 use strict;
 
-our $VERSION = '0.21';
+our $VERSION = '0.23';
 
 require XSLoader;
 XSLoader::load('Language::Lisp::ECLs', $VERSION);
@@ -13,6 +13,12 @@ sub new {
     return bless {}, __PACKAGE__;
 }
 
+sub shutdown {
+    print STDERR "1\n";
+    cl_shutdown();
+    print STDERR "2\n";
+}
+
 sub eval_string {
     my $self = shift;
     return _eval_string(@_);
@@ -27,6 +33,48 @@ sub stringify {
     return "{dummied stringification}";
 }
 
+#
+# AUTOLOAD method for lisp interpreter object, which will bring into
+# existance interpreter methods
+my %meth;
+sub AUTOLOAD {
+    my $int = shift;
+    my ($method,$method0,$package) = $Language::Lisp::ECLs::AUTOLOAD;
+    for ($method) {
+	s/^(Language::Lisp::ECLs::)//
+	    or die "weird inheritance ($method)";
+	$package = $1;
+        $method0 = $method;
+	s/(?<!_)__(?!_)/::/g;
+	s/(?<!_)___(?!_)/_/g;
+	# camelCase becomes camel-case
+	s/([a-z])(?=[A-Z])/$1-/g;
+    }
+    #print STDERR "AUTOLOAD($method,$package)\n";
+
+    if (!exists $meth{$method}) {
+	$meth{$method} = _search_lisp_function(uc($method));
+    }
+    my $code = $meth{$method};
+    unless (defined $code) {
+        print STDERR "code $method not found\n";
+	return;
+    }
+
+    # search for right corresponding lisp method, and create it afterwards
+    # (so no consequent AUTOLOAD will happen)
+    # TBD bind to fast subroutine
+    my $sub =  sub {
+	my $int = shift;
+	$code->funcall(@_);
+    };
+    no strict 'refs';
+    *{"$package$method0"} = $sub;
+    return $sub->($int,@_);
+}
+sub DESTROY {
+}
+
 package Language::Lisp::ECLs::Symbol;
 our @ISA = ('Language::Lisp::ECLs');
 package Language::Lisp::ECLs::Package;
@@ -105,6 +153,17 @@ same as eval_string but takes lisp object instead of string as argument.
 given lisp object blessed to package Language::Lisp::ECLs::Code calls the
 procedure.
 
+=head2 AUTOLOADing
+
+$cl->someFunctionName(args) get transformed into function call to
+"some-function-name"
+
+This is done by finding lisp object for evaluating arguments, and blessing
+it into Language::Lisp::ECLs::Code package
+
+  $cl->prin1("qwerty");
+  TODO
+
 =head2 EXPORT
 
 None. No namespace pollution, the greens are happy.
